﻿#include "AirportListParser.h"
#include "ProjectConfig.h"

#include <QDir>
#include <QFileInfo>
#include <QIODevice>
#include <QJsonObject>
#include <QJsonArray>
#include <QJsonDocument>
#include <QMessageBox>

#include <QtCoreKit/utils/QCKFile.h>

static bool LoadJsonObjectConfig(QJsonObject& json_obj, const std::string &file) 
{
	// Load File
	QByteArray file_data;
	if (QCKFile::LoadFile(file.c_str(), file_data) != QCKFile::State::Succeed) {
		QMessageBox::critical(NULL, __FUNCTION__, QString(file.c_str()) + " Not Exist!");
		return false;
	}

	QString json_str = file_data;
	QJsonDocument json_doc = QJsonDocument::fromJson(json_str.toUtf8());
	if (json_doc.isEmpty() || !json_doc.isObject()) {
		QMessageBox::critical(NULL, __FUNCTION__, QString(file.c_str()) + " parse json failed!");
		return false;
	}

	json_obj = json_doc.object();
	return true;
}

AirportListParser::AirportListParser()
{
	// parse json key-values
	QJsonObject json_obj;
	std::string cfg_file = ProjectConfig::GetInstance()->GetConfigFile("Config.json");
	if (LoadJsonObjectConfig(json_obj, cfg_file)) {
		if (json_obj.contains("airport_max_num")) {
			airport_max_num_ = json_obj["airport_max_num"].toInt();
		}
	}
}

bool AirportListParser::GetAirportList(std::vector<std::string>& list)
{
	list.clear();

	// parse json key-values
	QJsonObject json_obj;
	list_pool_file_ = ProjectConfig::GetInstance()->GetConfigFile("AirportList.json");
	if (!LoadJsonObjectConfig(json_obj, list_pool_file_)) {
		return false;
	}
	
	if (json_obj.contains("scenes") && json_obj["scenes"].isArray()) {
		QJsonArray json_arr = json_obj["scenes"].toArray();
		for (size_t idx = 0; idx < json_arr.size(); idx++) {
			if (json_arr[idx].isObject())
			{
				QJsonObject item = json_arr[idx].toObject();
				if (item.contains("name") && item["name"].toString() != u8"全局") {
					list.push_back(item["name"].toString().toStdString());
				}
			}
			//else if (json_arr[idx].isString())
			//{
			//	QString item = json_arr[idx].toString();
			//	list.push_back(item.toStdString());
			//}
		}
	}

	return true;
}

bool AirportListParser::SaveAirportList(const std::set<std::string>& list)
{
	// parse json key-values
	QJsonObject json_obj;
	list_pool_file_ = ProjectConfig::GetInstance()->GetConfigFile("AirportList.json");
	if (!LoadJsonObjectConfig(json_obj, list_pool_file_)) {
		return false;
	}

	if (json_obj.contains("scenes") && json_obj["scenes"].isArray()) {
		QJsonArray json_arr = json_obj["scenes"].toArray();
		for (size_t idx = 0; idx < json_arr.size(); idx++) {
			if (json_arr[idx].isObject())
			{
				QJsonObject item = json_arr[idx].toObject();
				if (item.contains("name") && item["name"].toString() != u8"全局") {
					auto name = item["name"].toString().toStdString();
					bool find_name = false;
					for (auto iter = list.begin(); iter != list.end(); iter++) {
						if ((*iter) == name) {
							find_name = true;
							break;
						}
					}

					if (!find_name) {
						json_arr.removeAt(idx);
						idx--;
					}
				}
			}
		}
		json_obj.remove("scenes");
		json_obj.insert("scenes", json_arr);
	}

	QString save_json_str = QString(QJsonDocument(json_obj).toJson());

	std::string save_file = ProjectConfig::GetInstance()->GetConfigFile("ExportList.vproj");
	return QCKFile::SaveFile(QString::fromStdString(save_file), save_json_str.toUtf8());
}
