﻿// MIT License
//
// Copyright (c) 2021~2022 [coder4869](https://github.com/coder4869)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

#include "QLandscapeDBDialog.h"
#include "AirportListParser.h"

#include <QDebug>
#include <QResizeEvent>
#include <QMessageBox>
#include <QStyleFactory>
#include <QtCoreKit/script/QCKCmd.h>
#include <QtCoreKit/utils/QCKMessage.h>

#include "ui_QLandscapeDBDialog.h"
#include "ProjectConfig.h"

QLandscapeDBDialog::QLandscapeDBDialog(QWidget *parent) :
    QDialog(parent), ui(new Ui::QLandscapeDBDialog)
{
    setWindowFlags(Qt::WindowMinMaxButtonsHint | Qt::WindowCloseButtonHint);
    setStyleSheet("QWidget { background-color: rgb(70, 76, 91); }");

    ui->setupUi(this);
    parser_ = std::make_shared<AirportListParser>();

    //ui->dockWidget->setWindowFlag(Qt::FramelessWindowHint);
    //ui->dockWidget->setFeatures(QDockWidget::NoDockWidgetFeatures);
    ui->listPoolWidget->setStyleSheet("\
                                    QListView::item{ height: 55px; border:0px; } \
                                    QListWidget::Item:hover{ color:white; } \
                                    QListWidget::item:selected{ color:rgb(26, 188, 156); } \
                                    ");
    connect(ui->listPoolWidget, SIGNAL(itemClicked(QListWidgetItem*)), this, SLOT(OnItemClicked(QListWidgetItem*)));
    
    ui->selectedListWidget->setStyle(QStyleFactory::create("windows"));
    ui->selectedListWidget->setStyleSheet("\
                                    QListView::item{ height: 55px; border:1px solid #797979; } \
                                    QListWidget::Item:hover{ color:white; } \
                                    QListWidget::item:selected{ color:rgb(26, 188, 156); border:1px solid #797979; } \
                                    ");
    QStringList list;
    for (size_t idx = 0; idx < parser_->GetAirportMaxNum(); idx++) {
        list << " ";
    }
    ui->selectedListWidget->addItems(list);
    connect(ui->selectedListWidget, SIGNAL(itemClicked(QListWidgetItem*)), this, SLOT(OnItemClicked(QListWidgetItem*)));

    ui->airportAddBtn->setStyleSheet("\
                                QPushButton{ border-radius:6px; background-color: white; width:150px; height:44px;} \
                                QPushButton::hover{ color:white; background-color: rgb(26, 188, 156); } \
                                ");
    connect(ui->airportAddBtn, SIGNAL(clicked(bool)), this, SLOT(OnAddAirport(bool)));

    ui->airportDelBtn->setStyleSheet("\
                                QPushButton{ border-radius:6px; background-color: white; width:150px; height:44px;} \
                                QPushButton::hover{ color:white; background-color: rgb(26, 188, 156); } \
                                ");
    connect(ui->airportDelBtn, SIGNAL(clicked(bool)), this, SLOT(OnDelAirport(bool)));

    ui->exportDataBtn->setStyleSheet("\
                                QPushButton{ border-radius:6px; color:white; background-color: #1ABC9C; width:150px; height:44px;} \
                                ");
    connect(ui->exportDataBtn, SIGNAL(clicked(bool)), this, SLOT(OnExportAirports(bool)));

    InitData();
    ListWidgetSortAndTextCenter(ui->listPoolWidget);
    ListWidgetSortAndTextCenter(ui->selectedListWidget);
}

QLandscapeDBDialog::~QLandscapeDBDialog()
{
    delete ui;
}

void QLandscapeDBDialog::InitData()
{
    std::vector<std::string> list;
    if (parser_->GetAirportList(list))
    {
        // Fill Data 
        for (size_t idx = 0; idx < list.size(); idx++) {
            auto text = QString(list[idx].c_str()).trimmed();
            ui->listPoolWidget->addItem(text);
        }
    }
}

void QLandscapeDBDialog::ListWidgetSortAndTextCenter(QListWidget* widget)
{
    // Sort and TextCenter
    //widget->setSortingEnabled(true);
    for (size_t idx = 0; idx < widget->count(); idx++)
    {
        widget->item(idx)->setTextAlignment(Qt::AlignCenter);
    }
}

void QLandscapeDBDialog::ListWidgetUpdateSelection(QListWidget* widget, QListWidgetItem* selected)
{
    bool has_selected = false;
    for (size_t idx = 0; idx < widget->count(); idx++)
    {
        if (widget->item(idx) == selected) {
            has_selected = true;
            break;
        }
    }

    if (!selected || !has_selected) {
        ListWidgetCancelSelection(widget);
    }
}

void QLandscapeDBDialog::ListWidgetCancelSelection(QListWidget* widget)
{
    for (size_t idx = 0; idx < widget->count(); idx++) {
        widget->item(idx)->setSelected(false);
    }
}

void QLandscapeDBDialog::resizeEvent(QResizeEvent* event)
{
    if (event->size() != event->oldSize()) {
        ui->dockWidget->resize(event->size());
    }
}


void QLandscapeDBDialog::OnItemClicked(QListWidgetItem* item)
{
    ListWidgetUpdateSelection(ui->listPoolWidget, item);
    ListWidgetUpdateSelection(ui->selectedListWidget, item);
}

void QLandscapeDBDialog::OnAddAirport(bool clicked) 
{
    ListWidgetCancelSelection(ui->selectedListWidget);

    QSet<QString> selected_list, keeped_list;
    // Already Selected
    for (size_t idx = 0; idx < ui->selectedListWidget->count(); idx++) {
        auto item = ui->selectedListWidget->item(idx);
        auto text = item->text().trimmed();
        if (!text.isEmpty()) {
            selected_list << text;
        }
        else { // Remove Empty
            ui->selectedListWidget->takeItem(idx);
            idx--;
        }
    }

    // Select Pool
    bool reach_max = false;
    for (size_t idx = 0; idx < ui->listPoolWidget->count(); idx++) {
        auto item = ui->listPoolWidget->item(idx);
        auto text = item->text().trimmed();
        if (!text.isEmpty()) {
            if (item->isSelected()) {
                if (selected_list.size() < parser_->GetAirportMaxNum()) { // Add
                    selected_list << text;
                    ui->selectedListWidget->addItem(text);
                }
                else { // Reach Max Limit : parser_->GetAirportMaxNum()
                    keeped_list << text;
                    reach_max = true;
                }
            }
            else { // Not Selected In Pool
                keeped_list << text;
            }
        }
        else { // Remove Empty
            ui->listPoolWidget->takeItem(idx);
            idx--;
        }
    }

    if (reach_max) {
        QString text = QString(u8"最多支持加载") + QString::number(parser_->GetAirportMaxNum()) + QString(u8"座机场!");
        //QCKMessage::ShowMsgBox(u8"提示", text);
        QMessageBox::warning(NULL, u8"提示", text);
    }
    else if (selected_list.size() < parser_->GetAirportMaxNum()) {
        for (size_t idx = selected_list.size(); idx < parser_->GetAirportMaxNum(); idx++) {
            ui->selectedListWidget->insertItem(idx, " ");
        }
    }

    ui->listPoolWidget->clear();
    QSetIterator<QString> iter(keeped_list);
    int idx = 0;
    while (iter.hasNext()) {
        ui->listPoolWidget->addItem(iter.next());
    }

    ListWidgetSortAndTextCenter(ui->listPoolWidget);
    ListWidgetSortAndTextCenter(ui->selectedListWidget);
}

void QLandscapeDBDialog::OnDelAirport(bool clicked)
{
    ListWidgetCancelSelection(ui->listPoolWidget);

    QSet<QString> selected_list, keeped_list;
    // Select Pool
    for (size_t idx = 0; idx < ui->listPoolWidget->count(); idx++) {
        auto item = ui->listPoolWidget->item(idx);
        auto text = item->text().trimmed();
        if (!text.isEmpty()) {
            keeped_list << text;
        }
        else { // Remove Empty
            ui->listPoolWidget->takeItem(idx);
            idx--;
        }
    }

    // Already Selected
    for (size_t idx = 0; idx < ui->selectedListWidget->count(); idx++) {
        auto item = ui->selectedListWidget->item(idx);
        auto text = item->text().trimmed();
        if (!text.isEmpty()) {
            if (item->isSelected()) {
                keeped_list << text; // Move to Select Pool
                ui->listPoolWidget->addItem(text);
            }
            else { // Not Selected In Pool
                selected_list << text;
            }
        }
        else { // Remove Empty
            ui->selectedListWidget->takeItem(idx);
            idx--;
        }
    }

    ui->selectedListWidget->clear();
    QSetIterator<QString> iter(selected_list);
    while (iter.hasNext()) {
        ui->selectedListWidget->addItem(iter.next());
    }

    if (selected_list.size() < parser_->GetAirportMaxNum()) {
        for (size_t idx = selected_list.size(); idx < parser_->GetAirportMaxNum(); idx++) {
            ui->selectedListWidget->insertItem(idx," ");
        }
    }

    ListWidgetSortAndTextCenter(ui->listPoolWidget);
    ListWidgetSortAndTextCenter(ui->selectedListWidget);
}

void QLandscapeDBDialog::OnExportAirports(bool clicked)
{
    ListWidgetCancelSelection(ui->listPoolWidget);

    QSet<QString> selected_list, keeped_list;
    // Already Selected
    for (size_t idx = 0; idx < ui->selectedListWidget->count(); idx++) {
        auto item = ui->selectedListWidget->item(idx);
        auto text = item->text().trimmed();
        if (!text.isEmpty()) {
            selected_list << text;
        }
    }

    if (selected_list.size() == 0) {
        QMessageBox::critical(NULL, u8"Export .vproj", "No Airport Selected!");
        return;
    }
    std::set<std::string> list;
    for (auto iter=selected_list.begin(); iter != selected_list.end(); iter++) {
        list.insert(iter->toStdString());
    }

    if (parser_->SaveAirportList(list)) {
        std::string bat_path = ProjectConfig::GetInstance()->GetToolFile("dispatch.bat");
        //"D:\Research\ViTools\ViConfigTool\bin64\tools"
        //"D:\Research\ViTools\ViConfigTool\bin64\tools\dispatch.bat"
        QByteArray output;
        bool ret = QCKCmd::ExecCmd(QString::fromStdString(bat_path), QStringList(), output);
        if (!ret) {
            qDebug() << QString(output);
            QMessageBox::critical(NULL, u8"Update .vproj", "Run tools/dispatch.bat failed ! Try it mannual with double click first!");
        }
        else {
            QMessageBox::information(NULL, u8"Update .vproj", u8"操作成功！");
        }
    }
}